@@ -25,27 +25,23 @@
 #' keybd.release('Alt')
 #' }
 keybd.press <- function(button, hold = FALSE) {
-  
-  if (!is.character(button)) {
-    stop("Argument 'button' must be a character string.")
+  if (!is.character(button)) stop("Argument 'button' must be a character string.")
+  raw_button_str <- strsplit(button, "\\+")[[1]]
+  button_str <- c()
+  for(i in raw_button_str){
+    if(i %in% keyboard_value$button && keyboard_value[keyboard_value$button==i,"shift"]){
+      warning(paste0("Button '",i,"' will be treated as lowercase. Use 'keybd.type_string' for uppercase keyboard simulation."))
+    }else if(!tolower(i) %in% keyboard_value$button){
+      stop(paste0("The 'button' value '", i, "' is not supported. Available keys are : ", paste0(as.character(keyboard_value$button), collapse = " ")))
+    }
+    button_str <- c(button_str, tolower(i))
   }
-  
-  button_str <- sapply(strsplit(button, "\\+")[[1]], FUN = tolower)
-  button_check <- !button_str %in% keyboard_value$button
-  if (any(button_check)) {
-    stop(paste0("The 'button' value '", button_str[button_check][1], "' is not supported. Available keys are : ", paste0(as.character(keyboard_value$button), collapse = " ")))
-  }
-  
-  idx <- match(button_str, keyboard_value$button)
-  button_virt_code <- keyboard_value[idx, ]$virt_code
-  button_scan_code <- keyboard_value[idx, ]$scan_code
-  button_scan_pref <- keyboard_value[idx, ]$prefix_byte
+  kv <- keyboard_value[match(button_str, keyboard_value$button), ]
   if (hold) {
-    press_c(as.vector(button_virt_code), as.vector(button_scan_code), as.vector(button_scan_pref))
+    press_c(kv$virt_code, kv$scan_code, kv$prefix_byte)
   } else {
-    press_and_release_c(as.vector(button_virt_code), as.vector(button_scan_code), as.vector(button_scan_pref))
+    press_and_release_c(kv$virt_code, kv$scan_code, kv$prefix_byte)
   }
-  
 }
 
 #' Simulate Key Release
@@ -66,23 +62,50 @@ keybd.press <- function(button, hold = FALSE) {
 #' keybd.release('Alt')
 #' }
 keybd.release <- function(button) {
-  
-  if (!is.character(button)) {
-    stop("Argument 'button' must be a character string.")
+  if (!is.character(button)) stop("Argument 'button' must be a character string.")
+  raw_button_str <- strsplit(button, "\\+")[[1]]
+  button_str <- c()
+  for(i in raw_button_str){
+    if(i %in% keyboard_value$button && keyboard_value[keyboard_value$button==i,"shift"]){
+      warning(paste0("Button '",i,"' will be treated as lowercase. Use 'keybd.type_string' for uppercase keyboard simulation."))
+    }else if(!tolower(i) %in% keyboard_value$button){
+      stop(paste0("The 'button' value '", i, "' is not supported. Available keys are : ", paste0(as.character(keyboard_value$button), collapse = " ")))
+    }
+    button_str <- c(button_str, tolower(i))
   }
-  
-  button_str <- sapply(strsplit(button, "\\+")[[1]], FUN = tolower)
+  kv <- keyboard_value[match(button_str, keyboard_value$button), ]
+  release_c(kv$virt_code, kv$scan_code, kv$prefix_byte)
+}
+
+#' Type a raw string
+#' 
+#' Type a raw string base on a given string.
+#' 
+#' @param string character. The string expected to output (case sensitive).
+#' @export
+#' @examples
+#' \dontrun{
+#' 
+#' # Type 'Hello world!'
+#' keybd.type_string("Hello world!")
+#' }
+keybd.type_string <- function(string) {
+  shift_key_row <- keyboard_value[keyboard_value$button=="shift",]
+  if (class(string) != 'character') stop("Argument 'string' must be a character string.")
+  button_str <- strsplit(string,"")[[1]]
   button_check <- !button_str %in% keyboard_value$button
   if (any(button_check)) {
     stop(paste0("The 'button' value '", button_str[button_check][1], "' is not supported. Available keys are : ", paste0(as.character(keyboard_value$button), collapse = " ")))
   }
-  
-  idx <- match(button_str, keyboard_value$button)
-  button_virt_code <- keyboard_value[idx, ]$virt_code
-  button_scan_code <- keyboard_value[idx, ]$scan_code
-  button_scan_pref <- keyboard_value[idx, ]$prefix_byte
-  release_c(as.vector(button_virt_code), as.vector(button_scan_code), as.vector(button_scan_pref))
-  
+  kv <- keyboard_value[match(button_str, keyboard_value$button), ]
+  for(i in 1:nrow(kv)){
+    if(kv[i,"shift"]){
+      combine_df <- rbind(shift_key_row,kv[i,])
+      press_and_release_c(combine_df$virt_code, combine_df$scan_code, combine_df$prefix_byte)
+    }else{
+      press_and_release_c(kv[i,"virt_code"], kv[i,"scan_code"], kv[i,"prefix_byte"])
+    }
+  }
 }
 
 # Mouse -------------------------------------------------------------------
@@ -105,7 +128,6 @@ keybd.release <- function(button) {
 #' mouse.click(button = "left", hold = TRUE)
 #' }
 mouse.click <- function(button = "left", hold = FALSE) {
-  
   btn <- tolower(button)
   if (btn == "left") {
     if (hold) {
@@ -122,7 +144,6 @@ mouse.click <- function(button = "left", hold = FALSE) {
   } else {
     stop("Argument 'button' should be either 'left' or 'right'.")
   }
-  
 }
 
 #' Simulate Mouse Click Release
@@ -171,9 +192,8 @@ mouse.release <- function(button = "left") {
 #' mouse.move(x=960,y=540,duration=3)
 #' }
 mouse.move <- function(x,y,duration=NA,step_ratio=0.01) {
-  if(class(x)!="numeric"|class(y)!="numeric"){
-    stop("Argument must be numeric")
-  }else if(!is.na(duration)){
+  if(class(x)!="numeric"|class(y)!="numeric") stop("Argument must be numeric")
+  if(!is.na(duration)){
     if(!is.numeric(duration)){
       stop("Argument must be numeric")
     }else if(class(step_ratio)!="numeric"){
